%
% 08/05-13
% This file is called automatically when simulation starts. It contains
% three pars. First part make a choke characteristic test that can be used
% to identify the choke characteristics. The second part loads biases and
% filters for the different measurements. The last part loads linear 4th
% order model and do the calculations for a Kalman filter and a LQR
% controller.

clear; clc;

%%%%%% Part 1: Make choke characteristic test %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Input Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
startValue = 90; % Maximum Choke Opening
endValue = 41; % Minium Choke Opening Resulting in 10 bar
slope = -0.1; % Slop of 
restTime = 30; % Time at startValue before starting decent
sample = 0.01; % Time between sample
testplot = 'n'; % Plot choke characteristic test, 'y' = yes and 'n' = no
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Calculation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n = 2*(restTime/sample + abs(startValue-endValue)/(abs(slope)*sample))+1;
endTime = n*sample-sample;
b = startValue-slope*restTime;

chokecharTest = zeros(n,1);
chokecharTime = (0:sample:endTime)';

for i=1:n
   if chokecharTime(i) <= restTime || chokecharTime(i) >= endTime-restTime
       chokecharTest(i) = startValue;
   else
      chokecharTest(i) = slope*chokecharTime(i)+b;
   end
   if chokecharTime(i) == ((n-1)/2)*sample
       slope = slope*-1;
       b = endValue-slope*chokecharTime(i);
   end
end

if strcmp(testplot,'y')
figure(1);
clf;
plot(chokecharTime,chokecharTest,'.');
title(sprintf('Length of series %4.1f',chokecharTime(end)));
end

% Tidy up
clearvars -except chokecharTest chokecharTime sample

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%% Part 2 Initialize filter and biases %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
name = 'biasFilter.mat'; % File containing biases and filtes
load(name)
clearvars name;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%% Part 3 Initialize LQG Controller and Kalman Filter %%%%%%%%%%%%%%%
% System:
% xdot = Ax + B1u + B2d
% y = Cx + D1u + D2d
% where x is state, u is the actuator signal, d is the disturbance and
% (A,B1,B2,C,D1,D2) are the system matrices. y is the measured signal.
%
% Controller
% u = -K(x-xstar)+ustar
% where [xstar;ustar] = inv([A B1;C D1])*([-B2;-D2]d+[0;1]r) and 
% r = y_desired. [xtar;ustar] = inv(M)*(N1*d+N2*r);

% For the lab u = choke pressure, d = [flow into well, piston refrence
% velocit] and  y = downhole pressure. 

%%%%%%% Inputs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load('linear4orderModel_exclDelay.mat') % Linear Model Identified from data
timeDelay = 0.72;
padeOrder = 1;
q1 = 10; % Weight on states in LQR controller
r1 = 1; % Weight on input in LQR controller
%%%%%%% Calculations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert to idss format and make continous and make state space form
C = CF/F;
sysd = idss(A,B,D,E,C,x0,sample);
sysc = d2c(sysd);
sys_ss = ss(sysc);

% Include delay in system
sysTF = tf(sysc);
[num,den] = pade(timeDelay,padeOrder);
delayTF = tf(num,den)*ones(1,3);
delayTF(2:3) = 1;
sysStar = ss(delayTF.*sysTF);

% B is a [n x 3 ] matrix where the first input is the controller signal,
% and the two others are considered disturbances.
inputs = 1; 

A = sysStar.a;
B = sysStar.b;
C = sysStar.c;
D = sysStar.d;
B1 = B(:,1:inputs);
B2 = B(:,inputs+1:end);
D1 = D(:,1:inputs);
D2 = D(:,inputs+1:end);

[modelOrder,~] = size(A);

% LQR controller
Q = q1*eye(modelOrder);
R = r1;

[K,S,E] = lqr(A,B1,Q,R,0);

% Matrices for calculation of ideal states
M = [A B1;C D1];
M_inv = inv(M);
N1 = [-B2;-D2];
N2 = [zeros(size(B2,1),1);ones(size(D2,1),1)];

% Kalman filter
[Kest,L,P] = kalman(sysStar,0.1,0.05);

